/*
 *	df3.c - primitive de gestion des "density files"
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "image.h"

/*::------------------------------------------------------------------::*/
#undef DEBUG_LEVEL
#define DEBUG_LEVEL 1

/*::------------------------------------------------------------------::*/
DensityF3Head * Image_df3_alloc(char *nom, int x, int y, int z, int b)
{
DensityF3Head	*ptrh;
double		*ptrd;		/* temporary ptr for data blob */
int		foo, nbcells;

#if DEBUG_LEVEL
fprintf(stderr, "*** allocating df3 descriptor\n");
fprintf(stderr, "sizeof(DensityF3Head) = %d\n", sizeof(DensityF3Head));
#endif

ptrh = calloc(1, sizeof(DensityF3Head));
if (NULL == ptrh)
	{
	fprintf(stderr, "%s:%s: no mem 1\n", __FILE__, __func__);
	exit(5);
	}

#if DEBUG_LEVEL
fprintf(stderr, "  header for df3 is at %p\n", ptrh);
#endif

ptrh->control = MAGIC_OF_DF3;
if (strlen(nom) > IMG_OBJNAME_LEN)
	strcpy(ptrh->name, "(invalid name)");
else
	strcpy(ptrh->name, nom);
ptrh->xdim = x;  ptrh->ydim = y;  ptrh->zdim = z;
ptrh->nbrbytes = b;
ptrh->flags = 0;
ptrh->scale = (double)1.0;

nbcells = x * y * z;
#if DEBUG_LEVEL
fprintf(stderr, "  %d cells in this df3 (double precision)\n", nbcells);
#endif

/* allocation of the data blob */
ptrd = malloc(nbcells * sizeof(double));
#if DEBUG_LEVEL
fprintf(stderr, "   data blob is at %p\n", ptrd);
#endif
for (foo=0; foo<nbcells; foo++)
	{
	ptrd[foo] = (double)0.0;
	}
ptrh->datas = ptrd;

strcpy(ptrh->blabla, "Plop !");

return ptrh;
}
/*::------------------------------------------------------------------::*/
int		Image_df3_free(DensityF3Head *ptr)
{

if (NULL == ptr)
	{
	fprintf(stderr, "%s: null pointer\n", __func__);
	return NULL_POINTER;
	}


return 0;
}
/*::------------------------------------------------------------------::*/
int Image_df3_set_bytes(DensityF3Head *ptr, int nbb)
{

if (NULL == ptr)
	{
	fprintf(stderr, "%s: null pointer\n", __func__);
	return NULL_POINTER;
	}
if (ptr->control != MAGIC_OF_DF3)
	{
	fprintf(stderr, "%s: %p bad magic\n", __func__, ptr);
	return VERY_STRANGE;
	}

if ( (nbb!=1) && (nbb!=2) && (nbb!=4) )
	{
	return INVALID_PARAM;
	}

ptr->nbrbytes = nbb;

return FUNC_NOT_FINISH;
}
/*::------------------------------------------------------------------::*/
int Image_df3_set_scale(DensityF3Head *ptr, double scale)
{

#if DEBUG_LEVEL
fprintf(stderr, "%s ( %p %g )\n", __func__, ptr, scale);
#endif

if (NULL == ptr)
	{
	fprintf(stderr, "%s: null pointer\n", __func__);
	return NULL_POINTER;
	}
if (ptr->control != MAGIC_OF_DF3)
	{
	fprintf(stderr, "%s: %p bad magic\n", __func__, ptr);
	return VERY_STRANGE;
	}

ptr->scale = scale;

return FUNC_NOT_FINISH;
}
/*::------------------------------------------------------------------::*/
int Image_df3_put(DensityF3Head *ptr, int x, int y, int z, double dv)
{
int	idx;

if ( (x<0) || (x>=ptr->xdim) ||
     (y<0) || (y>=ptr->ydim) ||
     (z<0) || (z>=ptr->zdim) )
	{
	printf("%s: out of bound\n", __func__);
	return INVALID_PARAM;
	}

idx = (x*ptr->ydim*ptr->zdim) + (y*ptr->zdim) + z;

#if DEBUG_LEVEL > 1
fprintf(stderr, "%s:  %p  (%d,%d,%d)-> %d <- %g\n", __func__,
				ptr, x, y, z, idx, dv);
#endif

ptr->datas[idx] = dv;

return FUNC_IS_BETA;
}
/*::------------------------------------------------------------------::*/
int Image_df3_get(DensityF3Head *ptr, int x, int y, int z, double *pdv)
{
int	idx;

if ( (x<0) || (x>=ptr->xdim) ||
     (y<0) || (y>=ptr->ydim) ||
     (z<0) || (z>=ptr->zdim) )
	{
	printf("%s: out of bound\n", __func__);
	return INVALID_PARAM;
	}

*pdv = 3.141592654;

return FUNC_IS_BETA;
}
/*::------------------------------------------------------------------::*/
int Image_df3f_get_dims(char *fname, int dims[], int display)
{
short	value;
FILE	*fpdf3;

#if DEBUG_LEVEL
fprintf(stderr, "lecture des dimensions de '%s'\n", fname);
#endif

if ( (fpdf3=fopen(fname, "rb")) == NULL )
	{
	fprintf(stderr, "%s: can't open '%s' for read\n", __func__, fname);
	return 666;
	}
Image_basicIO_read_M_short(fpdf3, &value);	dims[0] = (int)value;
Image_basicIO_read_M_short(fpdf3, &value);	dims[1] = (int)value;
Image_basicIO_read_M_short(fpdf3, &value);	dims[2] = (int)value;

if (display)
	{
	printf("+ df3 file '%s' -> %d %d %d\n", fname,
					dims[0], dims[1], dims[2]);
	}

fclose(fpdf3);

return FUNC_IS_BETA;
}
/*::------------------------------------------------------------------::*/
int Image_df3f_explique(char *fname)
{
int	foo, dims[3];

foo = Image_df3f_get_dims(fname, dims, 1);

return FUNC_NOT_FINISH;
}
/*::------------------------------------------------------------------::*/
int Image_df3_export(DensityF3Head *ptr, char *fname, int nbbytes)
{
FILE		*fp;
short		header[3];
int		idx, size;
double		dval;
U_BYTE		ub;
U_WORD		uw;
U_LONG		ul;

if ( (fp=fopen(fname, "wb")) == NULL )
	{
	fprintf(stderr, "%s: err fopen '%s'\n", __func__, fname);
	return FILE_CREATE_ERR;
	}

header[0] = ptr->xdim;
header[1] = ptr->ydim;
header[2] = ptr->zdim;
Image_basicIO_write_M_short(fp, header[0]);
Image_basicIO_write_M_short(fp, header[1]);
Image_basicIO_write_M_short(fp, header[2]);

size = ptr->xdim * ptr->ydim * ptr->zdim;
if (nbbytes == 0)
	nbbytes = ptr->nbrbytes;

fprintf(stderr, "... going to write %d cells of %d bytes.\n",
                                    size,         nbbytes);

for (idx=0; idx<size; idx++)
	{
	dval = ptr->datas[idx] * ptr->scale;
	switch (nbbytes)
		{
		case 1:
			ub = (int)(dval * 255.0);
			fwrite(&ub, 1, 1, fp);
			break;
		case 2:
			uw = (int)(dval * 65535.0);
			Image_basicIO_write_M_short(fp, uw);
			break;
		case 4:
			ul = (int)(dval * 255.0);
			fwrite(&ub, 1, 1, fp);
			break;

		default:	abort();

		}

	}

fclose(fp);

return FUNC_NOT_FINISH;
}
/*::------------------------------------------------------------------::*/
