/*
 *
	extrait de la documentation POV 3.1
     _________________________________________________________________
   
    Density_File
    
   The density_file pattern is a 3-D bitmap pattern that occupies a unit
   cube from location <0,0,0> to <1,1,1>. The data file is a raw binary
   file format created for POV-Ray called df3 format. The syntax provides
   for the possibility of implementing other formats in the future. This
   pattern was originally created for use with halo or media but it may
   be used anywhere any pattern may be used. The syntax is:
   
   pigment {density_file df3 "filename.df3" [interpolate Type]
          [PIGMENT_MODIFIERS...] }
          
   where "filename.df3" is a file name of the data file.
   
   As a normal pattern, the syntax is
   
   normal {density_file df3 "filename.df3" [, Bump_Size]
          [interpolate Type] [NORMAL_MODIFIERS...] 
          }
          
   The optional float Bump_Size should follow the file name and any other
   modifiers follow that.
   
   The df3 format consists of a 6 byte header of three 16-bit integers
   with high order byte first. These three values give the x,y,z size of
   the data in pixels (or more appropriately called voxels). This is
   followed by x*y*z unsigned integer bytes of data. The data in the
   range of 0 to 255 is scaled into a float value in the range 0.0 to
   1.0. It remains at 0.0 for all areas beyond the unit cube. The pattern
   occupies the unit cube regardless of the dimensions in voxels.
   
   The interpolate keyword may be specified to add interpolation of the
   data. The default value of zero specifies no interpolation. A value of
   one specifies tri-linear interpolation.
   
   See the sample scenes for data file include/spiral.df3,and the scenes
   which use it: scenes/textures/surfaces/densfile.pov,
   scenes/interior/media/galaxy.pov for examples.

 *
 */

#include  <stdio.h>
#include  <stdlib.h>
#include  "image.h"

/*::------------------------------------------------------------------::*/
/*
 		1) lire la doc, relire la doc
		2) comprendre la doc
		3) coder
 */
/*::------------------------------------------------------------------::*/

#define  DECALE		3
/* #define  COTE		(1<<DECALE) */
#define  COTE		32
#define  T_MATRICE	(COTE*COTE*COTE)
/*
 *	le paramtre "k" n'est pas utilis, pour le moment.
 */
int
Image_df3_xper_0(Image_Desc *src, char * fname_df3, int k)
{
short		header[3];
FILE		*fp;
int		idx, *calcul, maximum, minimum;
int	x, y, r, g, b, foo, bar, quux;
U_BYTE	ub;

if (src==NULL)
	{
	fprintf(stderr, "Aie, in df3_xper: src==NULL\n");
	return NULL_DESCRIPTOR;
	}

#if DEBUG_LEVEL
fprintf(stderr, "df3 xper_0: decalage  = %d\n", DECALE);
fprintf(stderr, "df3 xper_0: cote      = %d\n", COTE);
fprintf(stderr, "df3 xper_0: t matrice = %d\n", T_MATRICE);
#endif

if ( (fp=fopen(fname_df3, "wb")) == NULL )
	{
	fprintf(stderr, "df3 xper_0: err fopen '%s'\n", fname_df3);
	return FILE_CREATE_ERR;
	}

header[0] = header[1] = header[2] = COTE;
Image_basicIO_write_M_short(fp, header[0]);
Image_basicIO_write_M_short(fp, header[1]);
Image_basicIO_write_M_short(fp, header[2]);

if ( (calcul = calloc(T_MATRICE, sizeof(int))) == NULL )
	{
	fprintf(stderr, "df3 xper_0: no mem, Sorry\n");
	exit(5);
	}

#if DEBUG_LEVEL
fprintf(stderr, "calcul[] @ %p\n", calcul);
#endif

maximum = 0;
minimum = 999999999;
for (y=0; y<src->height; y++)
	{
	for (x=0; x<src->width; x++)
		{
		Image_getRGB(src, x, y, &r, &g, &b);
		r >>= DECALE;
		g >>= DECALE;
		b >>= DECALE;

		idx =	( r * (COTE*COTE) ) + 
			( g * (COTE) ) + 
			( b ) ;

		if (idx >= T_MATRICE)
			{
			fprintf(stderr, "DF3 overflow: %d\n", idx);
			exit(5);
			}

		foo = calcul[idx]++;
		if (foo > maximum)
			maximum = foo;
		if (foo < minimum)
			minimum = foo;
		}

#if DEBUG_LEVEL > 1
	printf("shift: %3d %3d %3d   %d\n", r, g, b, idx);
#endif
	}
fprintf(stderr, "df3 xper_0: maximum %d & minimum %d\n", maximum, minimum);

bar = 0;
for (foo=0; foo<T_MATRICE; foo++)
	{
	quux = ((calcul[foo] * 255) / maximum);
	ub = (U_BYTE)quux;

#if DEBUG_LEVEL > 1
	printf("%6d   %6d   %d\n", foo, calcul[foo], quux);
#endif
	fwrite(&ub, 1, 1, fp);
	if ( quux > bar )		bar = quux;
	}
fprintf(stderr, "df3 xper_0: le maximum crit est %d\n", bar);

free(calcul);
fclose(fp);

fprintf(stderr, "df3 xper_0: fini...\n");

return FUNC_IS_BETA;
}
/*::------------------------------------------------------------------::*/
#define BAS_HISTO	280
/*
 *	le paramtre "flag" n'est pas utilis, pour le moment.
 */
int
Image_df3f_plot_histo(char *fname_df3, char *nomtga, int flag)
{
Image_Desc	*img;
RGBA		paper, ink;
char		chaine[100];
int		foo, v, taille;
short		header[3];
long		histo[256], maxhist;
FILE		*fp;
U_BYTE		octet;

if ( (img = Image_alloc(400, 300, 3)) == NULL )
	{
	return IMAGE_NO_MEM;
	}
Image_clear(img, 0, 0, 0);

if ( (fp=fopen(fname_df3, "rb")) == NULL )
	{
	fprintf(stderr, "can't open '%s' for reading\n", fname_df3);
	return 666;
	}

#if DEBUG_LEVEL
fprintf(stderr, "Histo DF3 de %s\n", fname_df3);
#endif

Image_basicIO_read_M_short(fp, &header[0]);
Image_basicIO_read_M_short(fp, &header[1]);
Image_basicIO_read_M_short(fp, &header[2]);
fprintf(stderr, "%s:    %d  %d  %d\n", fname_df3,
				header[0], header[1], header[2]);
taille = header[0] * header[1] * header[2];
#if DEBUG_LEVEL
fprintf(stderr, "%d octets a lire\n", taille);
#endif

for (foo=0; foo<256; foo++)	histo[foo] = 0L;
for (foo=0; foo<taille; foo++)
	{
	fread(&octet, 1, 1, fp);
	histo[octet]++;
	}
maxhist = 0L;
for (foo=0; foo<256; foo++)
	if (histo[foo]>maxhist)	maxhist=histo[foo];
fprintf(stderr, "%s: maxhist=%ld\n", fname_df3, maxhist);

/* trac de la grille */
for (foo=0; foo<256; foo++)
	Image_plotRGB(img, foo+100, BAS_HISTO, 192, 192, 192);
for (foo=0; foo<256; foo+=4)
	{
	Image_plotRGB(img, foo+100, BAS_HISTO-128, 192, 192, 192);
	Image_plotRGB(img, foo+100, BAS_HISTO-256, 192, 192, 192);
	}

/* trac des points */
for (foo=0; foo<256; foo++)
	{
	v = (histo[foo]*255)/maxhist;
	Image_plotRGB(img, foo+100, BAS_HISTO-v, 255, 255, 0);
	}

/* criture de la lgende */
paper.r = paper.b = paper.g = 42;	paper.a = 0;
ink.r = ink.b = ink.g = 200;		ink.a = 255;
Image_trace_chaine_1(img, fname_df3, 5, 5, "libimage.fonte", &paper, &ink);

ink.b = 50;
sprintf(chaine, "x %4d", header[0]);
Image_trace_chaine_1(img, chaine, 5, 15, NULL, &paper, &ink);
sprintf(chaine, "y %4d", header[1]);
Image_trace_chaine_1(img, chaine, 5, 25, NULL, &paper, &ink);
sprintf(chaine, "z %4d", header[2]);
Image_trace_chaine_1(img, chaine, 5, 35, NULL, &paper, &ink);

Image_cadre_A(img);
#if DEBUG_LEVEL
fprintf(stderr, "saving df3 histo to %s\n", nomtga);
#endif
foo=Image_TGA_save(nomtga, img, 0);

Image_DeAllocate(img);		free(img);

return foo;
}
/*::------------------------------------------------------------------::*/
/*
 *	cette fonction devrait fabriquer un fichier parsable par awk.
 */
int
Image_df3f_2_txt(char *nomdf3, char *nomtxt)
{
FILE		*fpdf3, *fptxt;
U_WORD		header[3];
U_BYTE		octet;
int		taille,foo;

if ( (fpdf3=fopen(nomdf3, "rb")) == NULL )
	{
	fprintf(stderr, "df3 to txt: can't open '%s' for read\n", nomdf3);
	return 666;
	}

if ( (fptxt=fopen(nomtxt, "w")) == NULL )
	{
	fclose(fpdf3);
	fprintf(stderr, "df3 to txt: err fopen '%s'\n", nomtxt);
	return FILE_CREATE_ERR;
	}

Image_basicIO_read_M_short(fpdf3, (short *)&header[0]);
Image_basicIO_read_M_short(fpdf3, (short *)&header[1]);
Image_basicIO_read_M_short(fpdf3, (short *)&header[2]);
fprintf(stderr, "%s:    %d  %d  %d  -> %s\n", nomdf3,
			header[0], header[1], header[2], nomtxt);

taille = header[0] * header[1] * header[2];

for (foo=0; foo<taille; foo++)
	{
	fread(&octet, 1, 1, fpdf3);
	fprintf(fptxt, "%7d  %3d\n", foo, octet);
	}

fclose(fptxt);
fclose(fpdf3);

/*
 *	pour le moment, cette fonction crit un fichier qui me raconte
 *	que je ne comprends pas grand chose aux DF3s :(
 */

return FUNC_IS_ALPHA;
}
/*::------------------------------------------------------------------::*/
static int put_elem_s(FILE *fp, double x, double y, double z, double v)
{
int	foo;
foo = fprintf(fp, "sphere { <%f, %f, %f>, %f }\n", x, y, z, v * 0.5000);
return foo;
}

static int put_elem_b(FILE *fp, double x, double y, double z, double v)
{

return -1;
}

int
Image_df3f_2_povinc(char *nomdf3, char *nominc, char *nomobj, int k, double dv)
{
FILE		*fpdf3, *fpinc;
U_WORD		header[3];
U_BYTE		octet;
int		x, y, z, foo;
double		dx, dy, dz, poids;

#if DEBUG_LEVEL
fprintf(stderr, "%s:%s:\n\t", __FILE__, __func__);
fprintf(stderr, "nomdf3: %s   nominc: %s   k=%d dv=%g\n",
			nomdf3, nominc, k, dv);
#endif

if ( (fpdf3=fopen(nomdf3, "rb")) == NULL )
	{
	fprintf(stderr, "df3 to txt: can't open '%s' for read\n", nomdf3);
	return 666;	/* XXX */
	}

if ( (fpinc=fopen(nominc, "w")) == NULL )
	{
	fclose(fpdf3);
	fprintf(stderr, "df3 to inc: can't open '%s' for write\n", nominc);
	return FILE_CREATE_ERR;
	}

fprintf(fpinc, "/* GENERATED FILE, DO NOT EDIT BY HAND */\n\n");

Image_basicIO_read_M_short(fpdf3, (short *)&header[0]);
Image_basicIO_read_M_short(fpdf3, (short *)&header[1]);
Image_basicIO_read_M_short(fpdf3, (short *)&header[2]);
fprintf(stderr, "%s:    %d  %d  %d\n", nomdf3,
				header[0], header[1], header[2]);

fprintf(fpinc, "/* %s sizes: %d %d %d */\n",  nomdf3,
                                header[0], header[1], header[2]);

fprintf(fpinc, "#declare %s = object {\n", nomobj);
fprintf(fpinc, "union { \n");

for (x=0; x<header[0]; x++)
	{
	fprintf(fpinc, "/* X %d */\n", x);
	dx = (double)x;
	for (y=0; y<header[1]; y++)
		{
		dy = (double)y;
		for (z=0; z<header[2]; z++)
			{
			dz = (double)z;
			foo = fread(&octet, 1, 1, fpdf3);
			if (octet > 1)
				{
				poids = (double)octet / 256.0;
				put_elem_s(fpinc, dx, dy, dz, poids);
				}
			}
		}
#if DEBUG_LEVEL
	fflush(fpinc);
#endif
	}

fprintf(fpinc, " } \n } /* plop les moules */\n");

fclose(fpinc);

return FUNC_IS_ALPHA;
}
/*::------------------------------------------------------------------::*/

