/*
		IMG77A
		------

	interface pour le compilateur fortran GNU g77
	fonctions de base et/ou de support.

	il faut aussi aller voir le fichier 'img77.fi' qui tente
	de contenir les prototypes...

	la doc est ici: http://tth.zouh.org/image77.html

*/

#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	"image.h"

#include	"img77proto.h"

#define NB_SLOTS 42			/* a good number, no ? */

static  Image_Desc	*images[NB_SLOTS];

/*::------------------------------------------------------------------::*/
void
img_version__(int *pf)
{
Image_print_version(*pf);
fflush(stdout);
}
/*::------------------------------------------------------------------::*/
static int
cherche_slot(void)
{
int	foo;

/* on demarre a un pour detecter les numeros de
   slot mal initialise dans un appelant barbare. */

for (foo=1; foo<NB_SLOTS; foo++)
	{
	if (images[foo] == NULL)
		return foo;
#if DEBUG_LEVEL > 1
	else
		fprintf(stderr, "img77: slot %2d in use\n", foo);
#endif
	}
return -1;
}
/*::------------------------------------------------------------------::*/
/*	new 1 sept 2001							*/
Image_Desc * Image77_return_slot(int numslot)
{
#if DEBUG_LEVEL > 1
fprintf(stderr, "img77: requesting for slot #%d\n", numslot);
#endif

if (numslot < 1 || numslot >= NB_SLOTS)
	{
	fprintf(stderr, "img77: return slot: invalid slot %d\n", numslot);
#if FORCE_ABORT
	abort();
#endif
	exit(5);
	}
if (images[numslot] == NULL)
	{
	fprintf(stderr, "img77: return slot: invalid slot content\n");
#if FORCE_ABORT
	abort();
#endif
	exit(5);
	}

return images[numslot];
}
/*::------------------------------------------------------------------::*/
/*
 *	new 20 janvier 2003.
 *	semi-private function, only callable from C
 *	build for img_subimg proc in img77i.c
 */
int
Image77_addimg2slots(Image_Desc *img, int flag)
{
int	slot;

slot = cherche_slot();

if (slot < 0)
	{
	fprintf(stderr, "img77: addimg to slots: no slot available\n");
#if FORCE_ABORT
	abort();	/* kill, kill, kill */
#endif
	exit(5);	/* un peu brutal, mais bon... */
	}

if (flag && (slot > 0))
	{
	fprintf(stderr, "found a free slot: %d for %p\n", slot, img);
	}

images[slot] = img;

return slot;
}
/*::------------------------------------------------------------------::*/
void
img_dumpslots__(int *pf)
{
int	foo;
Image_Desc *img;

fflush(stdout);
printf("-----+ img77 :    dumping slots ---\n");

for (foo=1; foo<NB_SLOTS; foo++)
	{
	if ( images[foo]!=NULL )
		{
		img = images[foo];
		printf(" %3d | %8p ", foo, images[foo]);
		if (*pf)
			{
			printf("|  %5d x %5d  ", img->width, img->height);
			printf("  %-6s ", Image_type2str(img->type));
			printf("  %s", img->name);
			}
		puts("");
		}
	}
fflush(stdout);
}
/*::------------------------------------------------------------------::*/
int
img_create__(int *pw, int *ph, int *pm)
{
int		mode, slot;
Image_Desc	*image;

#if DEBUG_LEVEL > 1
fprintf(stderr, "img77: create(%d, %d, %d)\n", *pw, *ph, *pm);
#endif

if ((slot = cherche_slot()) < 1)
	{
	fprintf(stderr, "img77: create: no free slot found\n");
	exit(5);
	}

switch(*pm)
	{
	case 1:		mode = IMAGE_GRAY;	break;
	case 3:		mode = IMAGE_RGB;	break;
	case 4:		mode = IMAGE_RGBA;	break;
	default:	return -1;
	}

if ((image = Image_alloc(*pw, *ph, mode)) != NULL)
	{
#if DEBUG_LEVEL > 1
	fprintf(stderr, "img77: image allocated in slot %d ...\n", slot);
#endif
	images[slot] = image;
        strcpy(image->name, "(new)");
	return slot;
	}

return -1;
}
/*::------------------------------------------------------------------::*/
void
img_free__(int *pi)
{
int	foo;

#if DEBUG_LEVEL > 1
fprintf(stderr, "img77: attempting to free image #%d\n", *pi);
#endif

if (*pi < 1 || *pi >= NB_SLOTS)
	{
	fprintf(stderr, "img77: img_free: invalid slot number %d\n", *pi);
	exit(5);
	}
if (images[*pi] == NULL)
	{
	fprintf(stderr, "img77: img_free: invalid slot content (NULL)\n");
	exit(5);
	}
foo = Image_DeAllocate(images[*pi]);

free(images[*pi]);

images[*pi] = NULL;
}
/*::------------------------------------------------------------------::*/
/*
 *	le contenu "pixel" de l'image d'origine est copi
 */
int
img_clone__(int *pmodele)
{
Image_Desc	*modele, *clone;
int		slot;

if (*pmodele < 1 || *pmodele >= NB_SLOTS)
	{
	fprintf(stderr, "img77: img_clone: invalid slot %d\n", *pmodele);
	exit(5);
	}

modele = images[*pmodele];
if ( modele == NULL )
	{
	fprintf(stderr, "img77: img_clone: invalid slot content\n");
	exit(5);
	}

#if DEBUG_LEVEL > 1
Image_dump_descriptor(modele, "img77: image  cloner");
#endif

if ((slot = cherche_slot()) < 1)
	{
	fprintf(stderr, "IMG_CLONE: no free slot found\n");
	exit(5);
	}
/*
 *	beware, we copy model pixels in the cloned image !
 */
if ( (clone=Image_clone(modele, 1)) == NULL )
	{
	return 0;
	}

strcpy(clone->name, "(clone)");
images[slot] = clone;

#if DEBUG_LEVEL > 1
fprintf(stderr, "img_clone: slot of clone %p is %d\n", clone, slot);
#endif

return slot;
}
/*::------------------------------------------------------------------::*/
void
img_clear__(int *pi, int *pr, int *pg, int *pb)
{

if (*pi < 1 || *pi >= NB_SLOTS)
	{
	fprintf(stderr, "img77: img_clear: invalid slot number %d\n", *pi);
	exit(5);
	}
if (images[*pi] == NULL)
	{
	fprintf(stderr, "img77: img_clear: invalid slot content\n");
	exit(5);
	}

#if DEBUG_LEVEL > 1
fprintf(stderr, "img77: clear %d %d %d\n", *pr, *pg, *pb);
#endif

Image_clear(images[*pi], *pr, *pg, *pb);
}
/*::------------------------------------------------------------------::*/
void
img_dims__(int *pi, int *pw, int *ph)
{
Image_Desc	*image;
if (*pi < 1 || *pi >= NB_SLOTS)
	{
	fprintf(stderr, "img77: img_dims: invalid slot number %d\n", *pi);
	exit(5);
	}
image = images[*pi];
if (image->control != 0xdeadbeef)
	{
	fprintf(stderr, "img77: img_dims: no dead beef in %p\n", image);
	exit(5);
	}
*pw = image->width;	*ph = image->height;
}
/*::------------------------------------------------------------------::*/
/*
 *	il y a un controle des bordures de l'image dans cette fonction
 */
int
img_plot__(int *pi, int *px, int *py, int *pr, int *pg, int *pb)
{
Image_Desc *image;

if (*pi < 1 || *pi >= NB_SLOTS)
	{
	fprintf(stderr, "img77: plot: slot #%d invalid\n", *pi);
	exit(5);
	}

image = images[*pi];
if (image==NULL)
	{
	fprintf(stderr, "img77: plot: #%d non-existant image\n", *pi);
	exit(5);
	}

if ( *px<0 || *px>=image->width || *py<0 ||*py>=image->height )
	{
#if DEBUG_LEVEL
	fprintf(stderr, "img77: plot (%d,%d) out of image.\n", *px, *py);
#endif
	return 0;
	}

Image_plotRGB(image, *px, *py, *pr, *pg, *pb);

return 1;
}
/*::------------------------------------------------------------------::*/
int
img_plota__(int *pi, int *px, int *py, int *pr, int *pg, int *pb, int *pa)
{
Image_Desc *image;

if (*pi < 1 || *pi >= NB_SLOTS)
	{
	fprintf(stderr, "img77: plota: slot #%d invalid\n", *pi);
	exit(5);
	}

image = images[*pi];
if (image==NULL)
	{
	fprintf(stderr, "img77: plota: #%d non-existant image\n", *pi);
	exit(5);
	}

if ( *px<0 || *px>=image->width || *py<0 ||*py>=image->height )
	{
	fprintf(stderr, "img77: plota: (%d,%d) out of image.\n", *px, *py);
	return 0;
	}

Image_plotRGBA(image, *px, *py, *pr, *pg, *pb, *pa);

return 1;
}
/*::------------------------------------------------------------------::*/
int
img_rpix__(int *pi, int *px, int *py)
{
Image_Desc	*image;
image = images[*pi];
return Image_R_pixel(image, *px, *py);
}
/*::------------------------------------------------------------------::*/
int
img_gpix__(int *pi, int *px, int *py)
{
Image_Desc	*image;
image = images[*pi];
return Image_G_pixel(image, *px, *py);
}
/*::------------------------------------------------------------------::*/
int
img_bpix__(int *pi, int *px, int *py)
{
Image_Desc	*image;
image = images[*pi];
return Image_B_pixel(image, *px, *py);
}
/*::------------------------------------------------------------------::*/
int
img_rgb__(int *psrc, int *px, int *py, int *pr, int *pg, int *pb)
{
int	foo;
Image_Desc *s = images[*psrc];

foo = Image_getRGB(s, *px, *py, pr, pg, pb);

return foo;
}
/*::------------------------------------------------------------------::*/
int
img_draw__(int *pi, int *px1, int *py1, int *px2, int *py2,
						int *pr, int *pg, int *pb)
{
Image_Desc *image;
RGBA	rgba;
int	foo;

if (*pi < 1 || *pi >= NB_SLOTS)
	{
	fprintf(stderr, "img77: draw: image %d invalid\n", *pi);
	exit(5);
	}

image = images[*pi];
if (image==NULL)
	{
	fprintf(stderr, "img77: drawing in a non-existant image\n");
	exit(5);
	}

rgba.r = *pr;   rgba.g = *pg;   rgba.b = *pb;   rgba.a = 255;
foo = Image_draw_line(image, *px1, *py1, *px2, *py2, &rgba);

return 0;
}
/*::------------------------------------------------------------------::*/
/* remember: when you use this function in a F77 proggy, declare it
 * as returning an INTEGER ! */
int
tga_aload__(char *nom, int *pw, int *ph, int *pt, int hidden_l)
{
char		*filename, *ptr;
int		foo, slot;
Image_Desc	*image;

#if DEBUG_LEVEL > 1
printf("img77 aload: taille (hidden) de la chaine: %d\n", hidden_l);
#endif

/*
 *	kludge: recopier la chaine F77 pour en faire une chaine C
 */
if ((filename=(char *)malloc(hidden_l+1))==NULL)
	{
	fprintf(stderr, "img77: TGA_ALOAD: no memory\n");
	exit(5);
	}

foo = 0; 
ptr = filename;
while (foo<hidden_l && nom[foo] != ' ')
	{
	*ptr++ = nom[foo];
	++foo;
	}
*ptr = '\0';		/* 'fermer' la chaine C */

image = Image_TGA_alloc_load(filename);
if (image == NULL)
	{
	fprintf(stderr, "img77: TGA_ALOAD: '%s' loading error\n",
					filename);
	exit(5);
	}
slot = cherche_slot();

#if DEBUG_LEVEL > 1
fprintf(stderr, "img77: tga_aload: I give you the slot %d\n", slot);
#endif

if (slot < 0)
	{
	fprintf(stderr, "img77: TGA_ALOAD: no slot free, sorry.\n");
	exit(5);
	}

images[slot] = image;

strncpy(image->name, filename, IMG_OBJNAME_LEN);  /* XXX pourquoi */

*pw = image->width; 	*ph = image->height;
*pt = image->type;

return slot;
}
/*::------------------------------------------------------------------::*/
/*::------------------------------------------------------------------::*/
int
img_copy__(int *psrc, int *pdst)
{

if (*psrc < 1 || *psrc >= NB_SLOTS)
	{
	fprintf(stderr, "img77: copy: source image %d invalid\n", *psrc);
	exit(5);
	}
	
if (*pdst < 1 || *pdst >= NB_SLOTS)
	{
	fprintf(stderr, "img77: copy: destination image %d invalid\n", *pdst);
	exit(5);
	}

Image_copy(images[*psrc], images[*pdst]);
/* may be we can also copy the comment field ? */

return 0;
}
/*::------------------------------------------------------------------::*/
int
img_cadre__(int *pim, int *ptype)
{
Image_Desc *image;

if (*pim < 1 || *pim >= NB_SLOTS)
	{
	fprintf(stderr, "img77: cadre: image %d invalid\n", *pim);
	exit(5);
	}

image = images[*pim];
if (image==NULL)
	{
	fprintf(stderr, "img77: 'cadre' around non-existant image is silly\n");
	exit(5);
	}

switch (*ptype)
	{
	case 1:		Image_cadre_A(image);			break;
	case 2:		Image_cadre_cracra(image, 12, 0);	break;
	case 3:		Image_cadre_C(image, 0, 0, 0, 10);	break;
	case 4:		Image_cadre_E(image, 1, 2, 3, 10);	break;

	case 7:		Image_cadre_f(image, 64, 192);		break;
	case 8:		Image_cadre_f(image, 192, 64);		break;

	case 15:	Image_cadre_burp_0(image, 5, 5, 0);	break;
	case 17:	Image_cadre_burp_2(image, 5, 32, 64, 128); break;

	case 21:	Image_cadre_rasta_0(image, 10);		break;

	default:
		fprintf(stderr, "img77: cadre #%d not implemented\n", *ptype);
#if FORCE_ABORT
		abort();
#endif
		return UNKNOW_ERROR;
	}

return 0;
}
/*::------------------------------------------------------------------::*/
/*
	25 Sept 2000: Bon, voici une premiere tentative d'interface
	gnrique. 
*/
typedef struct
	{
	char	* keyword;
	int	function;
	} EFFECTS_KEYWORD;

EFFECTS_KEYWORD	effects_keywords[] =
	{
#define				EFF_WATER 1
	{	"water",	EFF_WATER	},

	{	NULL,		0		},

	};

#define NBRE_EFFECTS  (sizeof(effects_keywords)/sizeof(EFFECTS_KEYWORDS))

/* effective job begin here */

int
img_effects__(int *psrc, int *pdst, char *type, int *p1, int *p2, int *p3,
					int hidden_l)
{
char	*type_name, *ptr;
int	foo;

if ((type_name=(char *)malloc(hidden_l+2))==NULL)
	{
	fprintf(stderr, "img77: img_effect: no memory\n");
	exit(5);
	}

foo = 0; 
ptr = type_name;
while (foo<hidden_l && type[foo] != ' ')
	{
#if DEBUG_LEVEL
	printf("foo=%d  -> %c\n", foo, type[foo]);
#endif

	*ptr++ = type[foo];
	++foo;
	}
*ptr = '\0';		/* 'fermer' la chaine C */

#if DEBUG_LEVEL
fprintf(stderr, "img77: effect '%s'    %d -> %d\n",
			type_name, *psrc, *pdst);
#endif

/*
	maintenant, on cherche le 'mot clef' dans la table.
*/

return 0;
}
/*::------------------------------------------------------------------::*/
