/*
		IMG77D
		------
	POVray height_field specific functions.
	interface pour le compilateur fortran g77
*/

#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	"image.h"

#include	"img77proto.h"

/*::------------------------------------------------------------------::*/
Image_Desc * Image77_return_slot(int numslot);
/*::------------------------------------------------------------------::*/
/*
 * 	conversion rgb -> height-field
 */
void img_rgb2hf__(int *pi, int *pj)
{
Image_Desc *img, *dst;
int	foo;

img = Image77_return_slot(*pi);
dst = Image77_return_slot(*pj);

/* XXX check if the images have same dimensions ? */

foo = Image_hf15_rgb2hf(img, dst, 0);
Image_print_error("img77 rgb -> hf", foo);
}
/*::------------------------------------------------------------------::*/
/*
 *	ecriture d'une hauteur dans le HF.
 */
int
img_hf_plot__(int *pi, int *px, int *py, int *ph)
{
Image_Desc *img;
int	foo;

img = Image77_return_slot(*pi);

foo = Image_hf15_plot(img, *px, *py, *ph);

return 0;
}
/*::------------------------------------------------------------------::*/
/*
 *	lecture d'une hauteur - resultat entre 0 et 32767
 */
int
img_hf_height__(int *pi, int *px, int *py)
{
Image_Desc *img;
int	h;

img = Image77_return_slot(*pi);
/*
 *  que se passe-t-il si on cherche  lire en dehors de l'image ?
 */
h = Image_hf15_height(img, *px, *py);

return h;
}
/*::------------------------------------------------------------------::*/
/*
 *	incrmenter une hauteur aux coordonnes(x,y)
 */
int
img_hf_add__(int *pi, int *px, int *py, int *pah)
{
Image_Desc *img;
int	h, foo;

img = Image77_return_slot(*pi);
h = Image_hf15_height(img, *px, *py);
h += *pah;
foo = Image_hf15_plot(img, *px, *py, h);

return foo;				/* foo or h ? */
}
/*::------------------------------------------------------------------::*/
/*
 *	height_field smoothing (see pov_hf15.c)
 */
int
img_hf_smooth__(int *pi, int *pj, int *pk)
{
Image_Desc *src, *dst;
int	foo;

src = Image77_return_slot(*pi);
dst = Image77_return_slot(*pj);

#if DEBUG_LEVEL 
fprintf(stderr, "img77: lissage HF  %p -(%d)-> %p\n", src, *pk, dst);
#endif

foo = Image_hf15_lissage(src, dst, *pk, 0);

#if DEBUG_LEVEL > 1
fprintf(stderr, "img77: lissage HF  ret value is %d\n", foo);
#endif

return 0;
}
/*::------------------------------------------------------------------::*/
int
img_hf_norm__(int *pisrc, int *pidst, int *pmin, int *pmax)
{
Image_Desc *src, *dst;
int	foo;

if ( *pisrc<1 || *pidst<1 )
	{
	fprintf(stderr, "%s wrong slot %d %d\n", __func__, *pisrc, *pidst);
#if FORCE_ABORT
	abort();
#endif
	return 1;
	}

src = Image77_return_slot(*pisrc);
dst = Image77_return_slot(*pidst);

#if DEBUG_LEVEL
fprintf(stderr, "img77: normalise HF  %p -(%d,%d)-> %p\n",
				src, *pmin, *pmax, dst);
#endif

foo = Image_hf15_normalize(src, dst, *pmin, *pmax);
Image_print_error("img77: normalize HF", foo);

return 0;
}
/*::------------------------------------------------------------------::*/
/*
 * 		maybe, here, we can put a NOISE function ?
 * 	OK, 3 juillet 2009, une randomisation proportionnelle 
 * 	a la hauteur.
 */
void img_hf_noise3__(int *pslot, double *pprop, int *pflag)
{
Image_Desc	*img;
int		foo;

fprintf(stderr, "==== %s %d %f %d ====\n", __func__, *pslot, *pprop, *pflag);

img = Image77_return_slot(*pslot);

fprintf(stderr, "     img at %p\n", img);

foo = Image_hf15_noise_3(img, *pprop, *pflag);
fprintf(stderr, "     got a %d\n", foo);

}
/*::------------------------------------------------------------------::*/
/* new 7 dec 2009 */
void img_hf_sweep__(int *psrcslot, int *pdstslot, int *ptaille)
{
Image_Desc *src, *dst;
int	foo;

src = Image77_return_slot(*psrcslot);
dst = Image77_return_slot(*pdstslot);

fprintf(stderr, "==== %s  s=%d  d=%d  t=%d ====\n", __func__, 
					*psrcslot, *pdstslot, *ptaille);

foo = Image_hf15_sweep(src, 42, *ptaille, 0);

fprintf(stderr, "retour du sweep HF de #%d -> %d\n", *ptaille, foo);

/* void func */
}
/*::------------------------------------------------------------------::*/
/* new 15 dec 2009 - rastafortran coding @stex */
int img_hf_2pgm__(int *pslot, char *name, int szname)
{
Image_Desc	*img;
char		filename_buff[IMG_FILENAME_LEN+2];
int		foo;
char		* spacepos;

img = Image77_return_slot(*pslot);
if (NULL==img)
	{
	return 1;
	}

fprintf(stderr, "==== %s  s=%d i=%p ====\n", __func__, *pslot, img);

foo = IMG_FILENAME_LEN<szname ? IMG_FILENAME_LEN : szname;
strncpy(filename_buff, name, IMG_FILENAME_LEN);
filename_buff[foo] = '\0';
spacepos = strchr(filename_buff, ' ');
if (spacepos != NULL)
	{
#if DEBUG_LEVEL
	fprintf(stderr, "space at %p\n", spacepos);
#endif
	*spacepos = '\0';
	}

#if DEBUG_LEVEL > 1
for (foo=0; foo<19; foo++)
	fprintf(stderr, "%02x ", name[foo]);
fprintf(stderr, "\n");
for (foo=0; foo<19; foo++)
	fprintf(stderr, "%02x ", filename_buff[foo]);
fprintf(stderr, "\n");
#endif

foo = Image_hf15_save_PGM(filename_buff, img, __func__);

fprintf(stderr, "retour ecriture du HF en pgm -> %d\n", foo);

return foo;
}
/*::------------------------------------------------------------------::*/
