/*
 *		mircol.c
 *		========
 */

#include   <stdio.h>
#include   <stdlib.h>
#include   <time.h>
#include   <math.h>
#include   "image.h"

/*::------------------------------------------------------------------::*/
int
Image_mircol_0(Image_Desc *dst, char *txt, int flag)
{
int	x, y;
RGBA	fond, encre;

if (dst->width < 512 || dst->height < 512)
	{
	fprintf(stderr, "Image mircol 0: image %p < 512x512\n", dst);
	return IMAGE_TOO_SMALL;
	}

if (0 != flag)
	{
	fprintf(stderr, "%s -> flag is %d and must be 0\n", __func__, flag);
	}
if (NULL != txt)
	{
	fprintf(stderr, "%s : txt is '%s'\n", __func__, txt);
	}

for (x=0; x<256; x++)
	{
	for (y=0; y<256; y++)
		{
		Image_plotRGB(dst, x, y,	x, y, 0);
		Image_plotRGB(dst, x+256, y,	x, 0, y);
		Image_plotRGB(dst, x, y+256,	0, x, y);
		}
	}

for (x=0; x<256; x++)
	for (y=300; y<342; y++)
		Image_plotRGB(dst, x+256, y,	x, x, x);

fond.r = 0;	fond.g = 0;	fond.b = 0;	fond.a = 255;
encre.r = 220;	encre.g = 255;	encre.b = 255;	encre.a = 255;

if (txt != NULL)
	Image_trace_chaine_1(dst, txt, 270, 270, "libimage.fonte",
						&fond, &encre);

Image_trace_chaine_1(dst, "Libimage de tTh", 270, 370, "libimage.fonte",
						&fond, &encre);
Image_trace_chaine_1(dst, IMAGE_VERSION_STRING, 270, 385, NULL, &fond, &encre);
Image_trace_chaine_1(dst, __DATE__, 270, 400, NULL, &fond, &encre);

for (x=1; x<256; x++)
	for (y=420; y<462; y++)
		Image_plotRGB(dst, x+256, y,
			rand()%x, rand()%x, rand()%x);

return FUNC_IS_BETA;
}
/*::------------------------------------------------------------------::*/
int
Image_mircol_1(Image_Desc *dst, char *txt, int flag)
{
RGBA	fond, encre;
int	x, y, r, g, b;

if (dst->width < 512 || dst->height < 512)
	{
	fprintf(stderr, "Image mircol 1: image %p < 512x512\n", dst);
	return IMAGE_TOO_SMALL;
	}

for (x=0; x<512; x++)
	{
	for (y=0; y<512; y++)
		{
		r = x;
		g = (x + y) / 2;
		b = y;
		Image_plotRGB(dst, x, y, r, g, b);
		}
	}

fond.r = 0;	fond.g = 0;	fond.b = 0;	fond.a = 255;
encre.r = 255;	encre.g = 255;	encre.b = 255;	encre.a = 255;

Image_trace_chaine_1(dst, txt, 270, 270, "libimage.fonte", &fond, &encre);
Image_trace_chaine_1(dst, "Libimage de tTh", 270, 370, NULL, &fond, &encre);
Image_trace_chaine_1(dst, IMAGE_VERSION_STRING, 270, 390, NULL, &fond, &encre);

return FUNC_IS_ALPHA;
}
/*::------------------------------------------------------------------::*/
/* new 31 aout 2009 - ave StEx */
int Image_mircol_2(Image_Desc *dst, char *txt, int mode)
{
int	x, y, r, g, b, kx, ky;
float	fx, fy, fd;

#if DEBUG_LEVEL
fprintf(stderr, "%s ( %p '%s' %d )\n", __func__, dst, txt, mode);
#endif

for (y=0; y<dst->height; y++)
	{
	fy = (float)y / (float)dst->height;
	ky = (int)(fy * 256.0);
	for (x=0; x<dst->width; x++)
		{
		fx = (float)x / (float)dst->width;
		kx = (int)(fx * 256.0);
		r = kx;
		g = ky;
		fd = sqrt((fx*fx)+(fy*fy));
		b = (int)(fd * 256.0);
#if DEBUG_LEVEL > 1
		if (x==y)
			printf("%4d %4d   %7.3f %7.3f    %f\n",
					x, y, fx, fy, fd);
#endif
		Image_plotRGB(dst, x, y, r, g, b);
		}
	}

return FULL_NUCKED;
}
/*::------------------------------------------------------------------::*/
int
Image_mirRGB_0(Image_Desc *dst, int flag)
{
int	x, y;

#if DEBUG_LEVEL
fprintf(stderr, "%s : %p %d\n", __func__, dst, flag);
#endif

if (dst->width != 256 || dst->height != 256)
	{
	fprintf(stderr, "Image mire RGB 0: image is not 256x256\n");
	return IMAGE_BAD_DIMS;
	}

for (x=0; x<256; x++)
	{
	for (y=0; y<256; y++)
		{
		(dst->Rpix[x])[y] = x;
		(dst->Gpix[x])[y] = y;
		(dst->Bpix[x])[y] = (y+x)/2;
		}
	}

return FUNC_IS_BETA;
}
/*::------------------------------------------------------------------::*/
/* le bit 0 du flag controle l'ecriture des noms de composantes */
int
Image_decompose(Image_Desc *src, Image_Desc *dst, int flag)
{
int		foo, x, y, hx, hy;
Image_Desc	*moitie;
int		r, g, b;

if ( (foo=Image_compare_desc(src, dst)) )
	{
	fprintf(stderr, "%s: bad dimensions %d\n", __func__, foo);
	return foo;
	}

moitie = Image_MakeHalfSize(src, 1);
if (moitie == NULL)
	{
	fprintf(stderr, "Image Decompose: NO MEM\n");
	exit(5);
	}

if (0 != flag)
	fprintf(stderr, "%s : flag is 0x%X\n", __func__, flag);

hx = moitie->width;
hy = moitie->height;

for (x=0; x<moitie->width; x++)
	{
	for (y=0; y<moitie->height; y++)
		{
		Image_getRGB(moitie, x, y, &r, &g, &b);
		Image_plotRGB(dst, x, y, r, r, r);
		Image_plotRGB(dst, x+hx, y, g, g, g);
		Image_plotRGB(dst, x, y+hy, b, b, b);
		Image_plotRGB(dst, x+hx, y+hy, r, g, b);
		}
	}

/* faire le menage */
Image_DeAllocate(moitie);	free(moitie);

if (flag & 1)
	{
	RGBA	ink, pap;	/* hu ho ? local vars here ? */
	Image_load_fnt8x8("libimage.fonte", NULL, 0);
	pap.a = 0;	ink.a = 255;
	ink.r = 255;	ink.b = 0;	ink.g = 0;
	Image_trace_chaine_1(dst, "RED", 2, 2, NULL, &pap, &ink);
	ink.r = 0;	ink.g = 255;	ink.b = 0;
	Image_trace_chaine_1(dst, "GREEN", 2+hx, 2, NULL, &pap, &ink);
	ink.r = 10;	ink.g = 10;	ink.b = 255;
	Image_trace_chaine_1(dst, "BLUE", 2, 2+hy, NULL, &pap, &ink);
	}

return FUNC_IS_ALPHA;
}
/*::------------------------------------------------------------------::*/
