/*
			showdiff.c
			==========
Pour un front-edn, regardez http://tboudet.free.fr/libimage/img-outils.html
*/

#include  <stdio.h>
#include  <stdlib.h>
#include  <math.h>

#include  "image.h"

/*::------------------------------------------------------------------::*/
int
Image_showdiff_0(Image_Desc *im1, Image_Desc *im2, Image_Desc *dst)
{
int	x, y;
int	r1, g1, b1, r2, g2, b2;
int	dr, dg, db;
int	foo;

if ( (foo=Image_compare_desc(im1, im2)) )
    {
    fprintf(stderr, "Showdiff 0: src images have differents dims %d\n", foo);
    return foo;
    }
if ( (foo=Image_compare_desc(im1, dst)) )
    {
    fprintf(stderr, "Showdiff 0: dst: bad dimensions %d\n", foo);
    return foo;
    }

for (y=0; y<im1->height; y++)
	{
	for (x=0; x<im1->width; x++)
		{
		Image_getRGB(im1, x, y, &r1, &g1, &b1);
		Image_getRGB(im2, x, y, &r2, &g2, &b2);

		dr = ((r1 - r2)/2) + 127;
		dg = ((g1 - g2)/2) + 127;
		db = ((b1 - b2)/2) + 127;
		
		(dst->Rpix[y])[x] = dr;
		(dst->Gpix[y])[x] = dg;
		(dst->Bpix[y])[x] = db;
		}
	}

return OLL_KORRECT;
}
/*::------------------------------------------------------------------::*/
/* new 16 nov 2001
 *
 * parameter 'nu' is not used. put 0 in it.
 *
 *	this func provide an 'rgb-binary' image (utsl)
 *
 *	-----------------------------------------
 *	for component in (r, g, b)
 *		if component(im1)>component(im2)
 *		then pix=component.255
 *		else pix=component.0
 *	end for
 *	-----------------------------------------
 */
int
Image_showdiff_1(Image_Desc *im1, Image_Desc *im2, Image_Desc *dst, int nu)
{
int	x, y, r1, g1, b1, r2, g2, b2;
int	dr, dg, db, foo;

if ( (foo=Image_compare_desc(im1, im2)) )
    {
    fprintf(stderr, "Showdiff 1: src images differents dimensions %d\n", foo);
    return foo;
    }
if ( (foo=Image_compare_desc(im1, dst)) )
    {
    fprintf(stderr, "Showdiff 1: dst: bad dimensions %d\n", foo);
    return foo;
    }

for (y=0; y<im1->height; y++)
	{
	for (x=0; x<im1->width; x++)
		{
		/*Image_getRGB(im1, x, y, &r1, &g1, &b1);*/
		r1 = (im1->Rpix[y])[x];
		g1 = (im1->Gpix[y])[x];
		b1 = (im1->Bpix[y])[x];

		/*Image_getRGB(im2, x, y, &r2, &g2, &b2);*/
		r2 = (im2->Rpix[y])[x];
		g2 = (im2->Gpix[y])[x];
		b2 = (im2->Bpix[y])[x];

		dr = r1>r2 ? 0 : 255;
		dg = g1>g2 ? 0 : 255;
		db = b1>b2 ? 0 : 255;
		
		(dst->Rpix[y])[x] = dr;
		(dst->Gpix[y])[x] = dg;
		(dst->Bpix[y])[x] = db;
		}
	}
return OLL_KORRECT;
}
/*::------------------------------------------------------------------::*/
/*
 *	may be a scale problem in this function ?
 */
int
Image_showdiff_2(Image_Desc *im1, Image_Desc *im2, Image_Desc *dst, int dmin)
{
int	x, y;
int	r1, g1, b1, r2, g2, b2;
int	dr, dg, db;
int 	foo;

if ( (foo=Image_compare_desc(im1, im2)) )
    {
    fprintf(stderr, "Showdiff 2: src images differents dimensions %d\n", foo);
    return foo;
    }
if ( (foo=Image_compare_desc(im1, dst)) )
    {
    fprintf(stderr, "Showdiff 2: dst: bad dimensions %d\n", foo);
    return foo;
    }

for (y=0; y<im1->height; y++)
	{
	for (x=0; x<im1->width; x++)
		{
		Image_getRGB(im1, x, y, &r1, &g1, &b1);
		Image_getRGB(im2, x, y, &r2, &g2, &b2);

		dr = (r1*r1 - r2*r2)/256;
		dg = (g1*g1 - g2*g2)/256;
		db = (b1*b1 - b2*b2)/256;

		(dst->Rpix[y])[x] = dr;
		(dst->Gpix[y])[x] = dg;
		(dst->Bpix[y])[x] = db;
		}
	}

return FUNC_IS_BETA;
}
/*::------------------------------------------------------------------::*/
/*
 *	pour chaque composante
 *	   si delta(im1.C, im2.C) < kC
 *	      dst.C = 96
 *	   sinon
 *	      dst.C = 255
 */
int
Image_showdiff_3(Image_Desc *im1, Image_Desc *im2, Image_Desc *dst,
				int kr, int kg, int kb)
{
int	x, y, r1, g1, b1, r2, b2, g2;
int	dr, dg, db, r, g, b;

for (y=0; y<im1->height; y++)
	{
	for (x=0; x<im1->width; x++)
		{
		Image_getRGB(im1, x, y, &r1, &g1, &b1);
		Image_getRGB(im2, x, y, &r2, &g2, &b2);

		dr = abs(r1-r2);
		dg = abs(g1-g2);
		db = abs(b1-b2);

		if (dr<kr)	r = 0;
		else		r = 255;
		if (dg<kg)	g = 0;
		else		g = 255;
		if (db<kb)	b = 0;
		else		b = 255;

		(dst->Rpix[y])[x] = dr;
		(dst->Gpix[y])[x] = dg;
		(dst->Bpix[y])[x] = db;
		}
	}

dst->modified = 1;
return FUNC_IS_BETA;
}
/*::------------------------------------------------------------------::*/

